/* FichothequeLib_Xml - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.exportation;

import java.io.IOException;
import java.util.List;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.GroupClause;
import net.fichotheque.extraction.def.GroupParams;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.def.TitleClause;
import net.fichotheque.extraction.filterunit.AddendaExtractFilterUnit;
import net.fichotheque.extraction.filterunit.AlbumExtractFilterUnit;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.CorpusExtractFilterUnit;
import net.fichotheque.extraction.filterunit.DefaultIncludeUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.FicheRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MotcleRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.PhraseFilterUnit;
import net.fichotheque.extraction.filterunit.ThesaurusExtractFilterUnit;
import net.fichotheque.namespaces.BdfXmlSpace;
import net.fichotheque.selection.DocumentCondition;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.utils.FichothequeXMLUtils;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.xml.AppendableXMLPart;
import net.mapeadores.util.xml.AppendableXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDefXMLPart extends AppendableXMLPart {

    private final boolean compactStyle;

    public ExtractionDefXMLPart(AppendableXMLWriter xmlWriter, boolean compactStyle) {
        super(xmlWriter);
        this.compactStyle = compactStyle;
    }

    public void appendExtractionDef(ExtractionDef extractionDef) throws IOException {
        startOpenTag("extraction");
        addTagName(extractionDef.getTagNameInfo());
        endOpenTag();
        addDynamicCorpusExtract(extractionDef.getDynamicCorpusExtractDef());
        addDynamicThesaurusExtract(extractionDef.getDynamicThesaurusExtractDef());
        List<CorpusExtractDef> corpusExtractDefList = extractionDef.getStaticCorpusExtractDefList();
        List<ThesaurusExtractDef> thesaurusExtractDefList = extractionDef.getStaticThesaurusExtractDefList();
        if ((!corpusExtractDefList.isEmpty()) || (!thesaurusExtractDefList.isEmpty())) {
            startOpenTag("static");
            addTagName(extractionDef.getStaticTagNameInfo());
            endOpenTag();
            for (CorpusExtractDef corpusExtractDef : corpusExtractDefList) {
                addCorpusExtract(corpusExtractDef);
            }
            for (ThesaurusExtractDef thesaurusExtractDef : thesaurusExtractDefList) {
                addThesaurusExtract(thesaurusExtractDef);
            }
            closeTag("static");
        }
        closeTag("extraction");
    }

    private void addDynamicCorpusExtract(CorpusExtractDef corpusExtractDef) throws IOException {
        if (corpusExtractDef == null) {
            return;
        }
        if (canIgnore(corpusExtractDef)) {
            addFicheFilter(corpusExtractDef.getFicheFilter());
        } else {
            addCorpusExtract(corpusExtractDef);
        }
    }

    private void addCorpusExtract(CorpusExtractDef corpusExtractDef) throws IOException {
        String tagName = "fiches";
        List<FicheCondition.Entry> entryList = corpusExtractDef.getEntryList();
        String onlyCorpusAttribute = getOnlyCorpusAttribute(entryList);
        startOpenTag(tagName);
        addAttribute("name", corpusExtractDef.getName());
        if (onlyCorpusAttribute != null) {
            addAttribute("corpus", onlyCorpusAttribute);
        }
        addTagName(corpusExtractDef.getTagNameInfo());
        if (corpusExtractDef.isMaster()) {
            addAttribute("source", "master");
        }
        if (corpusExtractDef.isDescendantAxis()) {
            addAttribute("axis", "descendant");
        }
        endOpenTag();
        if (onlyCorpusAttribute == null) {
            for (FicheCondition.Entry entry : corpusExtractDef.getEntryList()) {
                openTag("fiche-query");
                FichothequeXMLUtils.writeFicheQuery(this, entry.getFicheQuery(), entry.getCroisementCondition());
                closeTag("fiche-query");
            }
        }
        if (corpusExtractDef.isHideEmpty()) {
            addEmptyElement("hide-empty");
        }
        addGroupClause(corpusExtractDef.getGroupClause());
        addPackClause(corpusExtractDef.getPackClause());
        addTitleClause(corpusExtractDef.getTitleClause());
        addFicheFilter(corpusExtractDef.getFicheFilter());
        closeTag(tagName);
    }

    private void addFicheFilter(FicheFilter ficheFilter) throws IOException {
        startOpenTag("fiche");
        addTagName(ficheFilter.getTagNameInfo());
        List<FilterUnit> filterUnitList = ficheFilter.getFilterUnitList();
        if (filterUnitList.isEmpty()) {
            closeEmptyTag();
        } else {
            endOpenTag();
            for (FilterUnit filterUnit : filterUnitList) {
                addFilterUnit(filterUnit);
            }
            closeTag("fiche");
        }
    }

    private void addDynamicThesaurusExtract(ThesaurusExtractDef thesaurusExtractDef) throws IOException {
        if (thesaurusExtractDef == null) {
            return;
        }
        addThesaurusExtract(thesaurusExtractDef);
    }

    private void addThesaurusExtract(ThesaurusExtractDef thesaurusExtractDef) throws IOException {
        String tagName = "motcles";
        startOpenTag(tagName);
        addAttribute("name", thesaurusExtractDef.getName());
        addTagName(thesaurusExtractDef.getTagNameInfo());
        if (thesaurusExtractDef.isMaster()) {
            addAttribute("source", "master");
        }
        endOpenTag();
        for (MotcleCondition.Entry entry : thesaurusExtractDef.getConditionEntryList()) {
            openTag("motcle-query");
            FichothequeXMLUtils.writeMotcleQuery(this, entry.getMotcleQuery(), entry.getCroisementCondition());
            closeTag("motcle-query");
        }
        addMotcleFilter(thesaurusExtractDef.getMotcleFilter(), null);
        closeTag(tagName);
    }

    private void addAddendaExtract(AddendaExtractDef addendaExtractDef) throws IOException {
        String tagName = "documents";
        startOpenTag(tagName);
        addAttribute("name", addendaExtractDef.getName());
        addTagName(addendaExtractDef.getTagNameInfo());
        endOpenTag();
        for (DocumentCondition.Entry entry : addendaExtractDef.getConditionEntryList()) {
            openTag("document-query");
            FichothequeXMLUtils.writeDocumentQuery(this, entry.getDocumentQuery(), entry.getCroisementCondition());
            closeTag("document-query");
        }
        DocumentFilter documentFilter = addendaExtractDef.getDocumentFilter();
        startOpenTag("document");
        addTagName(documentFilter.getTagNameInfo());
        closeEmptyTag();
        closeTag(tagName);
    }

    private void addAlbumExtract(AlbumExtractDef albumExtractDef) throws IOException {
        String tagName = "illustrations";
        startOpenTag(tagName);
        addAttribute("name", albumExtractDef.getName());
        addTagName(albumExtractDef.getTagNameInfo());
        endOpenTag();
        for (IllustrationCondition.Entry entry : albumExtractDef.getConditionEntryList()) {
            openTag("illustration-query");
            FichothequeXMLUtils.writeIllustrationQuery(this, entry.getIllustrationQuery(), entry.getCroisementCondition());
            closeTag("illustration-query");
        }
        IllustrationFilter illustrationFilter = albumExtractDef.getIllustrationFilter();
        startOpenTag("illustration");
        addTagName(illustrationFilter.getTagNameInfo());
        closeEmptyTag();
        closeTag(tagName);
    }

    private void addGroupClause(GroupClause groupClause) throws IOException {
        if (groupClause == null) {
            return;
        }
        startOpenTag("group");
        addAttribute("type", groupClause.getGroupType());
        GroupParams groupParams = groupClause.getGroupParams();
        if (groupParams != null) {
            addAttribute("params", groupParams.toString());
        }
        addAttribute("sort", groupClause.getSortOrder());
        addTagName(groupClause.getTagNameInfo());
        GroupClause subGroupClause = groupClause.getSubGroupClause();
        if (subGroupClause != null) {
            endOpenTag();
            addGroupClause(subGroupClause);
            closeTag("group");
        } else {
            closeEmptyTag();
        }
    }

    private void addPackClause(PackClause packClause) throws IOException {
        if (packClause == null) {
            return;
        }
        startOpenTag("pack");
        addAttribute("size", String.valueOf(packClause.getPackSize()));
        int modulo = packClause.getModulo();
        if (modulo > 0) {
            addAttribute("size", String.valueOf(modulo));
        }
        addTagName(packClause.getTagNameInfo());
        closeEmptyTag();
    }

    private void addTitleClause(TitleClause titleClause) throws IOException {
        if (titleClause == null) {
            return;
        }
        if (titleClause.withTitle()) {
            addEmptyElement("title");
        }
    }

    private void addFilterUnit(FilterUnit filterUnit) throws IOException {
        String params = toParamsString(filterUnit.getAttributes());
        if (filterUnit instanceof EnteteFilterUnit) {
            addEmptyElement("entete");
        } else if (filterUnit instanceof CorpsdeficheFilterUnit) {
            addEmptyElement("corpsdefiche");
        } else if (filterUnit instanceof FieldKeyFilterUnit) {
            FieldKeyFilterUnit fieldKeyFilterUnit = (FieldKeyFilterUnit) filterUnit;
            addFieldKey(fieldKeyFilterUnit.getFieldKey(), params);
        } else if (filterUnit instanceof FieldNamePrefixFilterUnit) {
            addFieldNamePrefix((FieldNamePrefixFilterUnit) filterUnit, params);
        } else if (filterUnit instanceof LotFilterUnit) {
            addLot((LotFilterUnit) filterUnit, params);
        } else if (filterUnit instanceof ChronoFilterUnit) {
            if (compactStyle) {
                appendCompact("chrono");
            } else {
                addEmptyElement("chrono");
            }
        } else if (filterUnit instanceof PhraseFilterUnit) {
            if (compactStyle) {
                appendCompact("phrase_");
                addText(((PhraseFilterUnit) filterUnit).getName());
            } else {
                startOpenTag("phrase");
                addAttribute("name", ((PhraseFilterUnit) filterUnit).getName());
                closeEmptyTag();
            }
        } else if (filterUnit instanceof AddendaExtractFilterUnit) {
            if (!insertDefaultIncludeUnit(filterUnit)) {
                addAddendaExtract(((AddendaExtractFilterUnit) filterUnit).getAddendaExtractDef());
            }
        } else if (filterUnit instanceof CorpusExtractFilterUnit) {
            if (!insertDefaultIncludeUnit(filterUnit)) {
                addCorpusExtract(((CorpusExtractFilterUnit) filterUnit).getCorpusExtractDef());
            }
        } else if (filterUnit instanceof ThesaurusExtractFilterUnit) {
            if (!insertDefaultIncludeUnit(filterUnit)) {
                addThesaurusExtract(((ThesaurusExtractFilterUnit) filterUnit).getThesaurusExtractDef());
            }
        } else if (filterUnit instanceof AlbumExtractFilterUnit) {
            if (!insertDefaultIncludeUnit(filterUnit)) {
                addAlbumExtract(((AlbumExtractFilterUnit) filterUnit).getAlbumExtractDef());
            }
        } else if (filterUnit instanceof FicheRattachementFilterUnit) {
            addFicheRattachement(((FicheRattachementFilterUnit) filterUnit).getFicheRattachementDef(), params);
        } else if (filterUnit instanceof MotcleRattachementFilterUnit) {
            addMotcleRattachement(((MotcleRattachementFilterUnit) filterUnit).getMotcleRattachementDef(), params);
        }
    }

    private boolean insertDefaultIncludeUnit(FilterUnit filterUnit) throws IOException {
        if (!compactStyle) {
            return false;
        }
        if (!(filterUnit instanceof DefaultIncludeUnit)) {
            return false;
        }
        appendIndent();
        append(((DefaultIncludeUnit) filterUnit).getExtendedIncludeKey().toString());
        return true;
    }

    private void addTagName(TagNameInfo tagNameInfo) throws IOException {
        switch (tagNameInfo.getType()) {
            case TagNameInfo.CUSTOM_TYPE:
                addAttribute("tag-name", tagNameInfo.getCustomTagName());
                break;
            case TagNameInfo.NULL_TYPE:
                addAttribute("tag-name", "_null");
                break;
        }
    }

    private void addFicheRattachement(FicheRattachementDef ficheRattachementDef, String params) throws IOException {
        FicheFilter ficheFilter = ficheRattachementDef.getFicheFilter();
        startOpenTag("fiche");
        List<SubsetKey> subsetKeyList = ficheRattachementDef.getRattachementSubsetKeyList();
        if (!subsetKeyList.isEmpty()) {
            StringBuilder buf = new StringBuilder();
            for (SubsetKey subsetKey : subsetKeyList) {
                if (buf.length() > 0) {
                    buf.append(',');
                }
                buf.append(subsetKey.getSubsetName());
            }
            addAttribute("corpus", buf.toString());
        }
        addTagName(ficheFilter.getTagNameInfo());
        addAttribute("params", params);
        List<FilterUnit> filterUnitList = ficheFilter.getFilterUnitList();
        if (filterUnitList.isEmpty()) {
            closeEmptyTag();
        } else {
            endOpenTag();
            for (FilterUnit filterUnit : filterUnitList) {
                addFilterUnit(filterUnit);
            }
            closeTag("fiche");
        }
    }

    private void addMotcleRattachement(MotcleRattachementDef motcleRattachementDef, String params) throws IOException {
        addMotcleFilter(motcleRattachementDef.getMotcleFilter(), params);
    }

    private void addMotcleFilter(MotcleFilter motcleFilter, String params) throws IOException {
        String tagName = getTagName(motcleFilter);
        startOpenTag(tagName);
        if (isRecursive(motcleFilter)) {
            addAttribute("recursive", "1");
        }
        addTagName(motcleFilter.getTagNameInfo());
        addAttribute("params", params);
        if (motcleFilter.isNoneFiltering()) {
            closeEmptyTag();
            return;
        }
        endOpenTag();
        if (motcleFilter.withLabels()) {
            addEmptyElement("labels");
        }
        if (motcleFilter.withLevel()) {
            addEmptyElement("level");
        }
        if (motcleFilter.withFicheStylePhrase()) {
            startOpenTag("phrase");
            addAttribute("name", FichothequeConstants.FICHESTYLE_PHRASE);
            closeEmptyTag();
        }
        MotcleFilter parentFilter = motcleFilter.getParentFilter();
        if (parentFilter != null) {
            if (!parentFilter.equals(motcleFilter)) {
                addMotcleFilter(parentFilter, null);
            }
        }
        MotcleFilter nextFilter = motcleFilter.getNextFilter();
        if (nextFilter != null) {
            addMotcleFilter(nextFilter, null);
        }
        MotcleFilter previousFilter = motcleFilter.getPreviousFilter();
        if (previousFilter != null) {
            addMotcleFilter(previousFilter, null);
        }
        MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
        if (childrenFilter != null) {
            if (!childrenFilter.equals(motcleFilter)) {
                addMotcleFilter(childrenFilter, null);
            }
        }
        for (FicheRattachementDef ficheRattachementDef : motcleFilter.getFicheRattachementDefList()) {
            addFicheRattachement(ficheRattachementDef, null);
        }
        List<CorpusExtractDef> corpusExtractDefList = motcleFilter.getIndexationCorpusExtractDefList();
        for (CorpusExtractDef corpusExtractDef : corpusExtractDefList) {
            addCorpusExtract(corpusExtractDef);
        }
        closeTag(tagName);
    }

    private boolean isRecursive(MotcleFilter motcleFilter) {
        boolean isRecursive = false;
        switch (motcleFilter.getType()) {
            case MotcleFilter.DEFAULT_TYPE: {
                MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
                if (childrenFilter != null) {
                    if (!childrenFilter.equals(motcleFilter)) {
                        isRecursive = true;
                    }
                }
                break;
            }
            case MotcleFilter.PARENT_TYPE: {
                MotcleFilter parentFilter = motcleFilter.getParentFilter();
                if (parentFilter != null) {
                    if (!parentFilter.equals(motcleFilter)) {
                        isRecursive = true;
                    }
                }
                break;
            }
        }
        return isRecursive;
    }

    private String getTagName(MotcleFilter motcleFilter) {
        switch (motcleFilter.getType()) {
            case MotcleFilter.NEXT_TYPE:
                return "next";
            case MotcleFilter.PARENT_TYPE:
                return "parent";
            case MotcleFilter.PREVIOUS_TYPE:
                return "previous";
            default:
                return "motcle";
        }
    }

    private void addFieldKey(FieldKey fieldKey, String params) throws IOException {
        if ((compactStyle) && (params == null)) {
            appendCompact(fieldKey.toString());
            return;
        }
        switch (fieldKey.getCategory()) {
            case FieldKey.PROPRIETE_CATEGORY:
                startOpenTag("propriete");
                addAttribute("name", fieldKey.getFieldName());
                break;
            case FieldKey.INFORMATION_CATEGORY:
                startOpenTag("information");
                addAttribute("name", fieldKey.getFieldName());
                break;
            case FieldKey.SECTION_CATEGORY:
                startOpenTag("section");
                addAttribute("name", fieldKey.getFieldName());
                break;
            case FieldKey.SPECIAL_CATEGORY:
                startOpenTag(fieldKey.getFieldString());
                break;
            default:
                throw new SwitchException("Unknown category = " + fieldKey.getCategory());
        }
        addAttribute("params", params);
        closeEmptyTag();
    }

    private void addFieldNamePrefix(FieldNamePrefixFilterUnit filterUnit, String params) throws IOException {
        short category = filterUnit.getCategory();
        startOpenTag(FieldKey.categoryToString(category));
        addAttribute("name", filterUnit.getPrefix() + "*");
        addAttribute("params", params);
        closeEmptyTag();
    }

    private void addLot(LotFilterUnit lotFilterUnit, String params) throws IOException {
        startOpenTag("lot");
        addTagName(lotFilterUnit.getTagNameInfo());
        addAttribute("params", params);
        endOpenTag();
        for (FilterUnit filterUnit : lotFilterUnit.getFilterUnitList()) {
            addFilterUnit(filterUnit);
        }
        closeTag("lot");
    }

    private String toParamsString(Attributes attributes) {
        if (attributes == null) {
            return null;
        }
        int count = attributes.size();
        if (count == 0) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(',');
            }
            Attribute attribute = attributes.get(i);
            AttributeKey attributeKey = attribute.getAttributeKey();
            if (attributeKey.getNameSpace().equals(BdfXmlSpace.EXTRACTION_NAMESPACE.toString())) {
                buf.append(attributeKey.getLocalKey());
            } else {
                buf.append(attributeKey.toString());
            }
            buf.append('=');
            buf.append(attribute.getFirstValue());
        }
        return buf.toString();
    }

    private void appendCompact(String text) throws IOException {
        appendIndent();
        append(text);
    }

    private boolean canIgnore(CorpusExtractDef corpusExtractDef) {
        if (corpusExtractDef.getTagNameInfo().getType() != TagNameInfo.NULL_TYPE) {
            return false;
        }
        if (corpusExtractDef.getName() != null) {
            return false;
        }
        if (corpusExtractDef.hasClauseObjects()) {
            return false;
        }
        if (corpusExtractDef.hasBooleanParameters()) {
            return false;
        }
        if (!corpusExtractDef.getEntryList().isEmpty()) {
            return false;
        }
        return true;
    }

    private String getOnlyCorpusAttribute(List<FicheCondition.Entry> entryList) {
        if (entryList.size() != 1) {
            return null;
        }
        FicheCondition.Entry entry = entryList.get(0);
        if (!SelectionUtils.isOnlyCorpusIncludeList(entry.getFicheQuery(), entry.getCroisementCondition())) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (SubsetKey corpusKey : entry.getFicheQuery().getCorpusCondition().getSubsetKeySet()) {
            if (buf.length() > 0) {
                buf.append(",");
            }
            buf.append(corpusKey.getSubsetName());
        }
        return buf.toString();
    }


}
